@@ -109,7 +109,7 @@ Options for image stretching / normalisation:
     * IMG_STRETCH_USER = 1
     * IMG_STRETCH_LINEAR = 2
     * IMG_STRETCH_STDEV = 3
-    * IMG_STRECTH_CUMULATIVE = 4
+    * IMG_STRETCH_CUMULATIVE = 4
 
 Options for vector geometry types:
 
@@ -256,7 +256,7 @@ IMG_STRETCH_NONE = 0
 IMG_STRETCH_USER = 1
 IMG_STRETCH_LINEAR = 2
 IMG_STRETCH_STDEV = 3
-IMG_STRECTH_CUMULATIVE = 4
+IMG_STRETCH_CUMULATIVE = 4
 
 THRES_METH_NONE = 0
 THRES_METH_OTSU = 1
